@@ -515,7 +515,6 @@ ptyxis_application_command_line (GApplication            *app,
           gtk_application_add_window (GTK_APPLICATION (self), GTK_WINDOW (window));
           gtk_window_present (GTK_WINDOW (window));
         }
-
     }
   else if (g_variant_dict_contains (dict, "tab"))
     {
@@ -584,7 +583,7 @@ ptyxis_application_command_line (GApplication            *app,
       g_application_activate (G_APPLICATION (self));
     }
 
-  return G_APPLICATION_CLASS (ptyxis_application_parent_class)->command_line (app, cmdline);
+  return EXIT_SUCCESS;
 }
 
 static void
@@ -792,6 +791,7 @@ ptyxis_application_startup (GApplication *application)
   g_autoptr(GError) error = NULL;
   AdwStyleManager *style_manager;
   gboolean sandbox_agent;
+  int timeout_msec;
 
   g_assert (PTYXIS_IS_APPLICATION (self));
 
@@ -814,11 +814,14 @@ ptyxis_application_startup (GApplication *application)
 
   G_APPLICATION_CLASS (ptyxis_application_parent_class)->startup (application);
 
-  sandbox_agent = ptyxis_application_should_sandbox_agent (self);
+  if ((sandbox_agent = ptyxis_application_should_sandbox_agent (self)))
+    timeout_msec = G_MAXINT;
+  else
+    timeout_msec = 2000;
 
   /* Try to spawn ptyxis-agent on the host when possible */
   if (!(self->client = ptyxis_client_new (sandbox_agent, &error)) ||
-      !ptyxis_client_ping (self->client, &error))
+      !ptyxis_client_ping (self->client, timeout_msec, &error))
     {
       self->client_is_fallback = TRUE;
 
@@ -834,7 +837,7 @@ ptyxis_application_startup (GApplication *application)
       g_clear_error (&error);
 
       if (!(self->client = ptyxis_client_new (TRUE, &error)) ||
-          !ptyxis_client_ping (self->client, &error))
+          !ptyxis_client_ping (self->client, G_MAXINT, &error))
         g_error ("Failed to spawn ptyxis-agent in sandbox: %s", error->message);
     }
 
@@ -1033,7 +1036,7 @@ ptyxis_application_init (PtyxisApplication *self)
     { "tab", 0, 0, G_OPTION_ARG_NONE, NULL, N_("New terminal tab in active window") },
     { "tab-with-profile", 0, 0, G_OPTION_ARG_STRING, NULL, N_("New terminal tab in active window using PROFILE"), N_("PROFILE") },
 
-    { "title", 0, 0, G_OPTION_ARG_STRING, NULL, N_("Set title for new tab") },
+    { "title", 'T', 0, G_OPTION_ARG_STRING, NULL, N_("Set title for new tab") },
 
     { NULL }
   };
