@@ -30,12 +30,13 @@
 #include "ptyxis-application.h"
 #include "ptyxis-profile-editor.h"
 #include "ptyxis-preferences-list-item.h"
+#include "ptyxis-util.h"
 
 struct _PtyxisProfileEditor
 {
   AdwNavigationPage  parent_instance;
 
-  PtyxisProfile    *profile;
+  PtyxisProfile     *profile;
 
   AdwEntryRow       *label;
   AdwSwitchRow      *bold_is_bright;
@@ -57,8 +58,8 @@ struct _PtyxisProfileEditor
   GtkScale          *opacity;
   GtkAdjustment     *opacity_adjustment;
   GtkLabel          *opacity_label;
-  AdwToastOverlay   *toasts;
-  GtkLabel          *uuid;
+  AdwSwitchRow      *use_proxy;
+  AdwActionRow      *uuid_row;
   GListStore        *erase_bindings;
   AdwComboRow       *backspace_binding;
   AdwComboRow       *delete_binding;
@@ -121,18 +122,60 @@ ptyxis_profile_editor_uuid_copy (GtkWidget  *widget,
 {
   PtyxisProfileEditor *self = (PtyxisProfileEditor *)widget;
   GdkClipboard *clipboard;
-  AdwToast *toast;
 
   g_assert (PTYXIS_IS_PROFILE_EDITOR (self));
 
   clipboard = gtk_widget_get_clipboard (widget);
   gdk_clipboard_set_text (clipboard, ptyxis_profile_get_uuid (self->profile));
 
-  toast = g_object_new (ADW_TYPE_TOAST,
-                        "title", _("Copied to clipboard"),
-                        "timeout", 3,
-                        NULL);
-  adw_toast_overlay_add_toast (self->toasts, toast);
+  gtk_widget_activate_action_variant (widget, "toast.add", ptyxis_variant_new_toast (_("Copied to clipboard"), 3));
+}
+
+static void
+undo_profile_delete (AdwToast      *toast,
+                     PtyxisProfile *profile)
+{
+  g_assert (ADW_IS_TOAST (toast));
+  g_assert (PTYXIS_IS_PROFILE (profile));
+
+  ptyxis_application_add_profile (PTYXIS_APPLICATION_DEFAULT, profile);
+}
+
+static void
+ptyxis_profile_editor_profile_delete (GtkWidget  *widget,
+                                      const char *action_name,
+                                      GVariant   *param)
+{
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
+  PtyxisProfileEditor *self = (PtyxisProfileEditor *)widget;
+  AdwPreferencesWindow *window;
+  AdwToast *toast;
+
+  g_assert (PTYXIS_IS_PROFILE_EDITOR (self));
+
+  g_object_ref (self);
+
+  window = ADW_PREFERENCES_WINDOW (gtk_widget_get_ancestor (widget, ADW_TYPE_PREFERENCES_WINDOW));
+  toast = adw_toast_new_format (_("Removed profile “%s”"),
+                                ptyxis_profile_dup_label (self->profile));
+  adw_toast_set_button_label (toast, _("Undo"));
+  g_signal_connect_data (toast,
+                         "button-clicked",
+                         G_CALLBACK (undo_profile_delete),
+                         g_object_ref (self->profile),
+                         (GClosureNotify)g_object_unref,
+                         0);
+
+  ptyxis_application_remove_profile (PTYXIS_APPLICATION_DEFAULT, self->profile);
+
+  adw_preferences_window_add_toast (window, toast);
+
+  adw_preferences_window_pop_subpage (window);
+
+  g_object_unref (self);
+
+G_GNUC_END_IGNORE_DEPRECATIONS
 }
 
 static char *
@@ -207,7 +250,7 @@ ptyxis_profile_editor_constructed (GObject *object)
   gsettings = ptyxis_profile_dup_settings (self->profile);
 
   g_object_bind_property (self->profile, "uuid",
-                          self->uuid, "label",
+                          self->uuid_row, "subtitle",
                           G_BINDING_SYNC_CREATE);
   g_object_bind_property (self->profile, "label",
                           self->label, "text",
@@ -233,6 +276,9 @@ ptyxis_profile_editor_constructed (GObject *object)
   g_object_bind_property (self->profile, "use-custom-command",
                           self->use_custom_commmand, "active",
                           G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
+  g_object_bind_property (self->profile, "use-proxy",
+                          self->use_proxy, "active",
+                          G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
   g_object_bind_property (self->profile, "custom-command",
                           self->custom_commmand, "text",
                           G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
@@ -421,13 +467,14 @@ ptyxis_profile_editor_class_init (PtyxisProfileEditorClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PtyxisProfileEditor, scroll_on_keystroke);
   gtk_widget_class_bind_template_child (widget_class, PtyxisProfileEditor, scroll_on_output);
   gtk_widget_class_bind_template_child (widget_class, PtyxisProfileEditor, scrollback_lines);
-  gtk_widget_class_bind_template_child (widget_class, PtyxisProfileEditor, toasts);
   gtk_widget_class_bind_template_child (widget_class, PtyxisProfileEditor, use_custom_commmand);
-  gtk_widget_class_bind_template_child (widget_class, PtyxisProfileEditor, uuid);
+  gtk_widget_class_bind_template_child (widget_class, PtyxisProfileEditor, use_proxy);
+  gtk_widget_class_bind_template_child (widget_class, PtyxisProfileEditor, uuid_row);
 
   gtk_widget_class_bind_template_callback (widget_class, get_container_title);
 
   gtk_widget_class_install_action (widget_class, "uuid.copy", NULL, ptyxis_profile_editor_uuid_copy);
+  gtk_widget_class_install_action (widget_class, "profile.delete", NULL, ptyxis_profile_editor_profile_delete);
 }
 
 static void
