@@ -32,9 +32,9 @@
 #include "ptyxis-application.h"
 #include "ptyxis-enums.h"
 #include "ptyxis-inspector.h"
-#include "ptyxis-tab.h"
 #include "ptyxis-tab-monitor.h"
 #include "ptyxis-tab-notify.h"
+#include "ptyxis-tab-private.h"
 #include "ptyxis-terminal.h"
 #include "ptyxis-util.h"
 #include "ptyxis-window.h"
@@ -79,6 +79,7 @@ struct _PtyxisTab
   PtyxisProcessLeaderKind  leader_kind : 3;
   guint                    has_foreground_process : 1;
   guint                    forced_exit : 1;
+  guint                    ignore_snapshot : 1;
 };
 
 enum {
@@ -99,6 +100,7 @@ enum {
 
 enum {
   BELL,
+  COMMIT,
   N_SIGNALS
 };
 
@@ -627,7 +629,17 @@ ptyxis_tab_dup_icon (PtyxisTab *self)
         const char *icon_name;
 
         if (!(container = ptyxis_tab_discover_container (self)))
-          g_set_object (&container, self->container_at_creation);
+          {
+            if (!g_set_object (&container, self->container_at_creation))
+              {
+                if (self->profile != NULL)
+                {
+                  g_autofree char *profile_uuid = ptyxis_profile_dup_default_container (self->profile);
+
+                  container = ptyxis_application_lookup_container (PTYXIS_APPLICATION_DEFAULT, profile_uuid);
+                }
+              }
+          }
 
         if (container != NULL &&
             (icon_name = ptyxis_ipc_container_get_icon_name (container)) &&
@@ -798,6 +810,9 @@ ptyxis_tab_snapshot (GtkWidget   *widget,
   g_assert (PTYXIS_IS_TAB (self));
   g_assert (GTK_IS_SNAPSHOT (snapshot));
 
+  if (self->ignore_snapshot)
+    return;
+
   window = PTYXIS_WINDOW (gtk_widget_get_root (widget));
   animating = ptyxis_window_is_animating (window);
   width = gtk_widget_get_width (widget);
@@ -903,6 +918,31 @@ ptyxis_tab_match_clicked_cb (PtyxisTab       *self,
   return FALSE;
 }
 
+static void
+ptyxis_tab_root (GtkWidget *widget)
+{
+  PtyxisTab *self = PTYXIS_TAB (widget);
+
+  /* Clear our ignore_snapshot bit in case we've had our tab restored
+   * from the parking lot.
+   */
+  self->ignore_snapshot = FALSE;
+
+  GTK_WIDGET_CLASS (ptyxis_tab_parent_class)->root (widget);
+}
+
+static void
+ptyxis_tab_commit_cb (PtyxisTab      *self,
+                      const char     *str,
+                      guint           length,
+                      PtyxisTerminal *terminal)
+{
+  g_assert (PTYXIS_IS_TAB (self));
+  g_assert (PTYXIS_IS_TERMINAL (terminal));
+
+  g_signal_emit (self, signals[COMMIT], 0, str);
+}
+
 static void
 ptyxis_tab_dispose (GObject *object)
 {
@@ -1053,6 +1093,7 @@ ptyxis_tab_class_init (PtyxisTabClass *klass)
   widget_class->map = ptyxis_tab_map;
   widget_class->snapshot = ptyxis_tab_snapshot;
   widget_class->size_allocate = ptyxis_tab_size_allocate;
+  widget_class->root = ptyxis_tab_root;
 
   properties[PROP_COMMAND_LINE] =
     g_param_spec_string ("command-line", NULL, NULL,
@@ -1137,6 +1178,17 @@ ptyxis_tab_class_init (PtyxisTabClass *klass)
                                 NULL,
                                 G_TYPE_NONE, 0);
 
+  signals[COMMIT] =
+    g_signal_new_class_handler ("commit",
+                                G_TYPE_FROM_CLASS (klass),
+                                G_SIGNAL_RUN_LAST,
+                                NULL,
+                                NULL, NULL,
+                                NULL,
+                                G_TYPE_NONE,
+                                1,
+                                G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
+
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Ptyxis/ptyxis-tab.ui");
   gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
   gtk_widget_class_set_css_name (widget_class, "ptyxistab");
@@ -1154,6 +1206,7 @@ ptyxis_tab_class_init (PtyxisTabClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, ptyxis_tab_bell_cb);
   gtk_widget_class_bind_template_callback (widget_class, ptyxis_tab_invalidate_icon);
   gtk_widget_class_bind_template_callback (widget_class, ptyxis_tab_match_clicked_cb);
+  gtk_widget_class_bind_template_callback (widget_class, ptyxis_tab_commit_cb);
 
   gtk_widget_class_install_action (widget_class, "tab.respawn", NULL, ptyxis_tab_respawn_action);
   gtk_widget_class_install_action (widget_class, "tab.inspect", NULL, ptyxis_tab_inspect_action);
@@ -1492,6 +1545,19 @@ ptyxis_tab_is_running (PtyxisTab  *self,
   return FALSE;
 }
 
+static gboolean
+ptyxis_tab_force_quit_in_idle (gpointer data)
+{
+  PtyxisTab *self = data;
+
+  g_assert (PTYXIS_IS_TAB (self));
+
+  if (self->process != NULL)
+    ptyxis_tab_send_signal (self, SIGKILL);
+
+  return G_SOURCE_REMOVE;
+}
+
 void
 ptyxis_tab_force_quit (PtyxisTab *self)
 {
@@ -1501,7 +1567,22 @@ ptyxis_tab_force_quit (PtyxisTab *self)
 
   self->forced_exit = TRUE;
 
-  ptyxis_tab_send_signal (self, SIGKILL);
+  if (self->process == NULL)
+    return;
+
+  /* First we try to send SIGHUP so that shells like bash will save their
+   * history (See #308).
+   */
+  ptyxis_tab_send_signal (self, SIGHUP);
+
+  /* In case this was not enough for the process to actually exit, we setup
+   * a short timer to send SIGKILL afterwards.
+   */
+  g_timeout_add_full (G_PRIORITY_LOW,
+                      50,
+                      ptyxis_tab_force_quit_in_idle,
+                      g_object_ref (self),
+                      g_object_unref);
 }
 
 PtyxisIpcProcess *
@@ -1811,7 +1892,8 @@ ptyxis_tab_open_uri_cb (GObject      *object,
   g_assert (G_IS_ASYNC_RESULT (result));
   g_assert (PTYXIS_IS_TAB (self));
 
-  if (!xdp_portal_open_uri_finish (XDP_PORTAL (object), result, &error))
+  if (!xdp_portal_open_uri_finish (XDP_PORTAL (object), result, &error) &&
+      !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
     ptyxis_tab_toast (self, 3, _("Failed to open link"));
 }
 
@@ -1919,3 +2001,11 @@ ptyxis_tab_query_working_directory_from_agent (PtyxisTab *self)
 
   return NULL;
 }
+
+void
+_ptyxis_tab_ignore_snapshot (PtyxisTab *self)
+{
+  g_return_if_fail (PTYXIS_IS_TAB (self));
+
+  self->ignore_snapshot = TRUE;
+}
