@@ -31,6 +31,7 @@
 #include "ptyxis-settings.h"
 #include "ptyxis-shrinker.h"
 #include "ptyxis-tab-monitor.h"
+#include "ptyxis-tab-private.h"
 #include "ptyxis-theme-selector.h"
 #include "ptyxis-title-dialog.h"
 #include "ptyxis-util.h"
@@ -75,6 +76,9 @@ struct _PtyxisWindow
   guint                  tab_overview_animating : 1;
   guint                  disposed : 1;
   guint                  single_terminal_mode : 1;
+  guint                  is_maximized : 1;
+  guint                  is_fullscreen : 1;
+  guint                  reset_nonvisible_from_size_allocate : 1;
 };
 
 G_DEFINE_FINAL_TYPE (PtyxisWindow, ptyxis_window, ADW_TYPE_APPLICATION_WINDOW)
@@ -129,6 +133,13 @@ ptyxis_window_close_page_dialog_cb (GObject      *object,
     }
 
   ptyxis_parking_lot_push (self->parking_lot, tab);
+
+  /* Ignore snapshot because libadwaita will try to snapshot this when
+   * calling adw_tab_view_close_page_finish(). This just skips past it
+   * until we maybe get re-added to a view later.
+   */
+  _ptyxis_tab_ignore_snapshot (tab);
+
   adw_tab_view_close_page_finish (self->tab_view, page, TRUE);
 
   /* Resize if we are going from 2->1 tabs */
@@ -775,6 +786,18 @@ ptyxis_window_active_tab_bell_cb (PtyxisWindow *self,
   ptyxis_window_visual_bell (self);
 }
 
+static void
+ptyxis_window_active_tab_commit_cb (PtyxisWindow *self,
+                                    const char   *str,
+                                    PtyxisTab    *tab)
+{
+  g_assert (PTYXIS_IS_WINDOW (self));
+  g_assert (PTYXIS_IS_TAB (tab));
+
+  if (self->is_fullscreen)
+    ptyxis_fullscreen_box_unreveal (self->fullscreen_box);
+}
+
 static void
 ptyxis_window_close_action (GtkWidget  *widget,
                             const char *action_name,
@@ -828,7 +851,9 @@ ptyxis_window_detach_action (GtkWidget  *widget,
 
   tab_page = adw_tab_view_get_page (self->tab_view, GTK_WIDGET (tab));
 
-  new_window = g_object_new (PTYXIS_TYPE_WINDOW, NULL);
+  new_window = g_object_new (PTYXIS_TYPE_WINDOW,
+                             "application", PTYXIS_APPLICATION_DEFAULT,
+                             NULL);
   adw_tab_view_transfer_page (self->tab_view, tab_page, new_window->tab_view, 0);
 
   gtk_window_present (GTK_WINDOW (new_window));
@@ -1033,6 +1058,7 @@ ptyxis_window_toplevel_state_changed_cb (PtyxisWindow *self,
 {
   GdkToplevelState state;
   gboolean is_fullscreen;
+  gboolean is_maximized;
 
   g_assert (PTYXIS_IS_WINDOW (self));
   g_assert (GDK_IS_TOPLEVEL (toplevel));
@@ -1040,11 +1066,31 @@ ptyxis_window_toplevel_state_changed_cb (PtyxisWindow *self,
   state = gdk_toplevel_get_state (toplevel);
 
   is_fullscreen = !!(state & GDK_TOPLEVEL_STATE_FULLSCREEN);
+  is_maximized = !!(state & GDK_TOPLEVEL_STATE_MAXIMIZED);
 
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "win.fullscreen", !is_fullscreen);
   gtk_widget_action_set_enabled (GTK_WIDGET (self), "win.unfullscreen", is_fullscreen);
 
   ptyxis_fullscreen_box_set_fullscreen (self->fullscreen_box, is_fullscreen);
+
+  /* Clear cached grid size for non-visible tabs when leaving fullscreen or
+   * maximized state. Otherwise we'll jump back to large grid size which is
+   * not expected from a user-standpoint.
+   */
+  if ((!is_maximized && self->is_maximized) ||
+      (!is_fullscreen && self->is_fullscreen))
+    self->reset_nonvisible_from_size_allocate = TRUE;
+
+  /* If transitioning to fullscreen, animate in the fullscreen controls which
+   * ensures that they will disappear after timeout.
+   *
+   * See: https://gitlab.gnome.org/chergert/ptyxis/-/issues/376
+   */
+  if (!self->is_fullscreen && is_fullscreen)
+    ptyxis_fullscreen_box_reveal (self->fullscreen_box);
+
+  self->is_fullscreen = is_fullscreen;
+  self->is_maximized = is_maximized;
 }
 
 static void
@@ -1419,6 +1465,30 @@ ptyxis_window_update_menu_visibility (PtyxisWindow *self)
   gtk_widget_set_visible (GTK_WIDGET (self->new_terminal_menu_button), visible);
 }
 
+static void
+notify_decoration_layout_cb (PtyxisWindow *self,
+                             GParamSpec   *pspec,
+                             GtkSettings  *gtk_settings)
+{
+  g_autofree char *layout = NULL;
+  gboolean inverted = FALSE;
+  const char *colon;
+  const char *close_;
+
+  g_assert (PTYXIS_IS_WINDOW (self));
+  g_assert (GTK_IS_SETTINGS (gtk_settings));
+
+  g_object_get (gtk_settings,
+                "gtk-decoration-layout", &layout,
+                NULL);
+
+  if ((colon = strchr (layout, ':')) && (close_ = strstr (layout, "close")))
+    inverted = close_ < colon;
+
+  if (self->tab_bar)
+    adw_tab_bar_set_inverted (self->tab_bar, inverted);
+}
+
 static void
 ptyxis_window_constructed (GObject *object)
 {
@@ -1429,6 +1499,7 @@ ptyxis_window_constructed (GObject *object)
   g_autoptr(GMenuModel) profile_menu = NULL;
   g_autoptr(GMenuModel) container_menu = NULL;
   g_autoptr(GMenu) menu = NULL;
+  GtkSettings *gtk_settings;
 
   G_OBJECT_CLASS (ptyxis_window_parent_class)->constructed (object);
 
@@ -1468,6 +1539,14 @@ ptyxis_window_constructed (GObject *object)
                            self,
                            G_CONNECT_SWAPPED);
   ptyxis_window_update_menu_visibility (self);
+
+  gtk_settings = gtk_settings_get_default ();
+  g_signal_connect_object (gtk_settings,
+                           "notify::gtk-decoration-layout",
+                           G_CALLBACK (notify_decoration_layout_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+  notify_decoration_layout_cb (self, NULL, gtk_settings);
 }
 
 static void
@@ -1508,6 +1587,39 @@ ptyxis_window_my_computer_action (GtkWidget  *widget,
   gtk_widget_activate_action (widget, "win.new-terminal", "(ss)", "", "session");
 }
 
+static void
+ptyxis_window_size_allocate (GtkWidget *widget,
+                             int        width,
+                             int        height,
+                             int        baseline)
+{
+  PtyxisWindow *self = (PtyxisWindow *)widget;
+
+  g_assert (PTYXIS_IS_WINDOW (self));
+
+  GTK_WIDGET_CLASS (ptyxis_window_parent_class)->size_allocate (widget, width, height, baseline);
+
+  if (self->reset_nonvisible_from_size_allocate)
+    {
+      g_autoptr(GListModel) pages = ptyxis_window_list_pages (self);
+      guint n_pages = g_list_model_get_n_items (pages);
+
+      for (guint i = 0; i < n_pages; i++)
+        {
+          g_autoptr(AdwTabPage) page = g_list_model_get_item (pages, i);
+          PtyxisTab *tab = PTYXIS_TAB (adw_tab_page_get_child (page));
+          PtyxisTerminal *terminal = ptyxis_tab_get_terminal (tab);
+
+          if (page == adw_tab_view_get_selected_page (self->tab_view))
+            continue;
+
+          ptyxis_terminal_reset_for_size (terminal);
+        }
+
+      self->reset_nonvisible_from_size_allocate = FALSE;
+    }
+}
+
 static void
 ptyxis_window_dispose (GObject *object)
 {
@@ -1596,6 +1708,7 @@ ptyxis_window_class_init (PtyxisWindowClass *klass)
   object_class->set_property = ptyxis_window_set_property;
 
   widget_class->realize = ptyxis_window_realize;
+  widget_class->size_allocate = ptyxis_window_size_allocate;
 
   window_class->close_request = ptyxis_window_close_request;
 
@@ -1708,6 +1821,11 @@ ptyxis_window_init (PtyxisWindow *self)
                                  G_CALLBACK (ptyxis_window_active_tab_bell_cb),
                                  self,
                                  G_CONNECT_SWAPPED);
+  g_signal_group_connect_object (self->active_tab_signals,
+                                 "commit",
+                                 G_CALLBACK (ptyxis_window_active_tab_commit_cb),
+                                 self,
+                                 G_CONNECT_SWAPPED);
   g_signal_group_connect_object (self->active_tab_signals,
                                  "notify::process-leader-kind",
                                  G_CALLBACK (ptyxis_window_notify_process_leader_kind_cb),
